module Language.Mecha.Types
  ( Vector, Vertex, Normal, Color
  , Moveable  (..)
  , Scaleable (..)
  , Colorable (..)
  , Setable   (..)
  , moveX
  , moveY
  , moveZ
  , scaleAll
  , unions
  , debug
  ) where

import System.IO.Unsafe

type Vector = (Double, Double, Double)
type Vertex = Vector
type Normal = Vector
type Color  = Vector

class Moveable a where
  move    :: Vector -> a -> a
  rotateX :: Double -> a -> a
  rotateY :: Double -> a -> a
  rotateZ :: Double -> a -> a

moveX :: Moveable a => Double -> a -> a
moveX a = move (a, 0, 0)

moveY :: Moveable a => Double -> a -> a
moveY a = move (0, a, 0)

moveZ :: Moveable a => Double -> a -> a
moveZ a = move (0, 0, a)

class Scaleable a where
  scale :: Vector -> a -> a

scaleAll :: Scaleable a => Double -> a -> a
scaleAll a = scale (a, a, a)

class Colorable a where
  color :: Color -> a -> a

class Setable a where
  union        :: a -> a -> a
  intersection :: a -> a -> a
  difference   :: a -> a -> a

unions :: Setable a => [a] -> a
unions = foldl1 union

debug :: Show a => String -> a -> b -> b
debug m a b = unsafePerformIO (putStrLn (m ++ ": " ++ show a) >> return b)
